-------------------------------------------------------------------------------
-- Title      : Silicon Labs Si549 oscillator I2C controller
-- Project    : White Rabbit
-------------------------------------------------------------------------------
-- File       : wr_si549_interface.vhd
-- Author     : Tomasz Wlostowski
-- Company    : CERN BE-Co-HT
-- Created    : 2010-04-26
-- Platform   : FPGA-generic
-- Standard   : VHDL '93
-------------------------------------------------------------------------------
-- Author     : Jorge Machado
-- Created    : 2021-03-12
-- Platform   : FPGA-generic
-- Standard   : VHDL '93
-------------------------------------------------------------------------------
-- Description: Silabs Si549 series oscillator hardware interface for the WR
-- Core SoftPLL, allowing it to discipline Si549 oscillators to WR clock.
-- The core performs two tasks:
-- - allows the WR core to access I2C registers of attached Si549 oscillator
--   through bit-banging (e.g. to set base frequency)
-- - controls the Si549 PLL tuning in atomic (HW-only) way via tm_dac_value port.
--   The tuning value is added to pre-programmed RFREQ bias value set via
--   Wishbone.
-------------------------------------------------------------------------------
--
-- Copyright (c) 2013 CERN
--
-- This source file is free software; you can redistribute it   
-- and/or modify it under the terms of the GNU Lesser General   
-- Public License as published by the Free Software Foundation; 
-- either version 2.1 of the License, or (at your option) any   
-- later version.                                               
--
-- This source is distributed in the hope that it will be       
-- useful, but WITHOUT ANY WARRANTY; without even the implied   
-- warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      
-- PURPOSE.  See the GNU Lesser General Public License for more 
-- details.                                                     
--
-- You should have received a copy of the GNU Lesser General    
-- Public License along with this source; if not, download it   
-- from http://www.gnu.org/licenses/lgpl-2.1.html
--
-------------------------------------------------------------------------------

library ieee;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.wishbone_pkg.all;
use work.si549_wbgen2_pkg.all;

entity wr_si549_interface is
  port (
    clk_sys_i : in std_logic;
    rst_n_i   : in std_logic;

    -- WR Core timing interface: aux clock tune port
    tm_dac_value_i    : in std_logic_vector(23 downto 0) := x"000000";
    tm_dac_value_wr_i : in std_logic := '0';

    -- I2C bus: output enable (active low)
    scl_pad_oen_o : out std_logic;
    sda_pad_oen_o : out std_logic;

    -- I2C bus: input pads
    scl_pad_i : in std_logic;
    sda_pad_i : in std_logic;

    -- Wishbone
    wb_adr_i   : in  std_logic_vector(c_wishbone_address_width-1 downto 0)   := (others => '0');
    wb_dat_i   : in  std_logic_vector(c_wishbone_data_width-1 downto 0)      := (others => '0');
    wb_dat_o   : out std_logic_vector(c_wishbone_data_width-1 downto 0);
    wb_sel_i   : in  std_logic_vector(c_wishbone_address_width/8-1 downto 0) := (others => '0');
    wb_we_i    : in  std_logic                                               := '0';
    wb_cyc_i   : in  std_logic                                               := '0';
    wb_stb_i   : in  std_logic                                               := '0';
    wb_ack_o   : out std_logic;
    wb_err_o   : out std_logic;
    wb_stall_o : out std_logic
    );

end wr_si549_interface;

architecture rtl of wr_si549_interface is

  component si549_if_wb
    port (
      rst_n_i    : in  std_logic;
      clk_sys_i  : in  std_logic;
      wb_adr_i   : in  std_logic_vector(2 downto 0);
      wb_dat_i   : in  std_logic_vector(31 downto 0);
      wb_dat_o   : out std_logic_vector(31 downto 0);
      wb_cyc_i   : in  std_logic;
      wb_sel_i   : in  std_logic_vector(3 downto 0);
      wb_stb_i   : in  std_logic;
      wb_we_i    : in  std_logic;
      wb_ack_o   : out std_logic;
      wb_stall_o : out std_logic;
      regs_i     : in  t_si549_in_registers;
      regs_o     : out t_si549_out_registers);
  end component;

  signal regs_in  : t_si549_in_registers;
  signal regs_out : t_si549_out_registers;

  signal fbdiv_new_p                            : std_logic;
  signal fbdiv_base, fbdiv_current, fbdiv_new : unsigned(42 downto 0);

  signal lsdiv : std_logic_vector(2 downto 0);
  signal hsdiv : std_logic_vector(10 downto 0);
  

  signal tm_dac_value_wr_d : std_logic;


  
  signal i2c_tick    : std_logic;
  signal i2c_divider : unsigned(7 downto 0);

  signal scl_int : std_logic;
  signal sda_int : std_logic;

  signal seq_count : unsigned(8 downto 0);
  signal fdiv_adj_scaled            : signed(23 downto 0);
  signal fdiv_adj_scaled_current    : signed(23 downto 0);

  type t_i2c_transaction is (START, STOP, SEND_BYTE);

  type t_state is (
  IDLE, SI_START0, SI_START1, SI_START2, SI_START3, SI_START4, SI_START5, SI_START6, SI_START7,
  SI_ADDR0, SI_ADDR1, SI_ADDR2, SI_ADDR3, SI_ADDR4, SI_ADDR5, SI_ADDR6,  SI_ADDR7,
  SI_STOP0, SI_STOP1, SI_STOP2, SI_STOP3, SI_STOP4, SI_STOP5, SI_STOP6, SI_STOP7,
  SI_REG255_ADDR, SI_REG231_ADDR, SI_PAGE, SI_REG69_ADDR, SI_DIS_FCAL, SI_REG17_ADDR, SI_DIS_OUTPUT,
  SI_REG23_ADDR, SI_REG23, SI_REG24, SI_REG26, SI_REG27, SI_REG28, SI_REG29, SI_REG30, SI_REG31,
  SI_REG07_ADDR, SI_START_FCAL, SI_REG17_ADDR_2, SI_EN_OUTPUT, SI_REG26_ADDR, ADPLL_DELTA_M_0, 
  ADPLL_DELTA_M_1, ADPLL_DELTA_M_2);

  signal state : t_state;

  signal scl_out_host, scl_out_fsm : std_logic;
  signal sda_out_host, sda_out_fsm : std_logic;
  
  attribute mark_debug : string;
  attribute mark_debug of state : signal is "TRUE";
  attribute mark_debug of lsdiv : signal is "TRUE";  
  attribute mark_debug of hsdiv : signal is "TRUE";
  attribute mark_debug of fbdiv_new_p : signal is "TRUE";
  attribute mark_debug of fbdiv_base : signal is "TRUE";
  attribute mark_debug of fbdiv_current : signal is "TRUE";
  attribute mark_debug of fbdiv_new : signal is "TRUE";
  attribute mark_debug of tm_dac_value_wr_i : signal is "TRUE";
  attribute mark_debug of fdiv_adj_scaled : signal is "TRUE";
  attribute mark_debug of seq_count : signal is "TRUE";
  attribute mark_debug of scl_pad_oen_o : signal is "TRUE";
  attribute mark_debug of sda_pad_oen_o : signal is "TRUE";
  attribute mark_debug of scl_pad_i : signal is "TRUE";
  attribute mark_debug of sda_pad_i : signal is "TRUE";
  attribute mark_debug of tm_dac_value_i : signal is "TRUE";
  attribute mark_debug of fdiv_adj_scaled_current : signal is "TRUE";
  attribute mark_debug of regs_out : signal is "TRUE";
  

  procedure f_i2c_iterate(tick : std_logic; signal counter : inout unsigned; value : std_logic_vector(7 downto 0); trans_type : t_i2c_transaction; signal scl : out std_logic; signal sda : out std_logic; signal state_var : out t_state; next_state : t_state) is
    variable last : boolean;
  begin

    last := false;

    if(tick = '0') then
      return;
    end if;


    case trans_type is
      when START =>
        case counter(1 downto 0) is
          -- states 0..2: start condition
          when "00" =>
            scl <= '1';
            sda <= '1';
          when "01" =>
            sda <= '0';
          when "10" =>
            scl  <= '0';
            last := true;
          when others => null;
        end case;

      when STOP =>
        case counter(1 downto 0) is
          -- states 0..2: start condition
          when "00" =>
            sda <= '0';
          when "01" =>
            scl <= '1';
          when "10" =>
            sda  <= '1';
            last := true;
          when others => null;
        end case;
        
      when SEND_BYTE =>
        
        case counter(1 downto 0) is
          when "00" =>
            sda <= value(7-to_integer(counter(4 downto 2)));
          when "01" =>
            scl <= '1';
          when "10" =>
            scl <= '0';
            if(counter(5) = '1') then
              last := true;
            end if;
          when others => null;
        end case;
    end case;

    if(last) then
      state_var <= next_state;
      counter   <= "000000000";
    else
      counter <= counter + 1;
    end if;
    
  end f_i2c_iterate;


  function f_sign_extend( x : signed; l : integer ) return unsigned is
    variable rv : unsigned(l-1 downto 0);
  begin
    rv ( x'length-1 downto 0) := unsigned(x);
    rv( l-1 downto x'length ) := (others => x(x'length-1));
    return rv;
  end f_sign_extend;
  
  
  signal resized_addr : std_logic_vector(c_wishbone_address_width-1 downto 0);

  signal wb_in  : t_wishbone_slave_in;
  signal wb_out : t_wishbone_slave_out;
  
begin  -- rtl

  resized_addr(4 downto 0)                          <= wb_adr_i(4 downto 0);
  resized_addr(c_wishbone_address_width-1 downto 5) <= (others => '0');

  U_Adapter : wb_slave_adapter
    generic map(
      g_master_use_struct  => true,
      g_master_mode        => CLASSIC,
      g_master_granularity => WORD,
      g_slave_use_struct   => false,
      g_slave_mode         => PIPELINED,
      g_slave_granularity  => BYTE)
    port map (
      clk_sys_i  => clk_sys_i,
      rst_n_i    => rst_n_i,
      master_i   => wb_out,
      master_o   => wb_in,
      sl_adr_i   => resized_addr,
      sl_dat_i   => wb_dat_i,
      sl_sel_i   => wb_sel_i,
      sl_cyc_i   => wb_cyc_i,
      sl_stb_i   => wb_stb_i,
      sl_we_i    => wb_we_i,
      sl_dat_o   => wb_dat_o, 
      sl_ack_o   => wb_ack_o,
      sl_stall_o => wb_stall_o);

  U_WB_Slave : si549_if_wb
    port map (
      rst_n_i    => rst_n_i,
      clk_sys_i  => clk_sys_i,
      wb_adr_i   => wb_in.adr(2 downto 0),
      wb_dat_i   => wb_in.dat,
      wb_dat_o   => wb_out.dat,
      wb_cyc_i   => wb_in.cyc,
      wb_sel_i   => wb_in.sel,
      wb_stb_i   => wb_in.stb,
      wb_we_i    => wb_in.we,
      wb_ack_o   => wb_out.ack,
      wb_stall_o => wb_out.stall,
      regs_i     => regs_in,
      regs_o     => regs_out);

  regs_in.cr_busy_i <= '1' when state /= IDLE else '0';

  fbdiv_base(31 downto 0)  <= unsigned(regs_out.fdiv_fbfrac_o);
  fbdiv_base(42 downto 32) <= unsigned(regs_out.fdiv_fbint_o(10 downto 0));

  lsdiv <= regs_out.lsdiv_o(2 downto 0);
  hsdiv <= regs_out.hsdiv_o(10 downto 0);

  p_rfreq : process(clk_sys_i)
  begin
    if rising_edge(clk_sys_i) then
      if rst_n_i = '0' then
        fbdiv_new_p <= '0';
      else
        if(tm_dac_value_wr_i = '1') then
          fdiv_adj_scaled <= (signed(tm_dac_value_i(15 downto 0)) - to_signed(32768, 16)) * signed(regs_out.cr_gain_o);
        end if;
        
       fbdiv_new_p <= tm_dac_value_wr_i and regs_out.cr_enable_o;
      end if;
    end if;
  end process;

  p_i2c_divider : process(clk_sys_i)
  begin
    if rising_edge(clk_sys_i) then
      if rst_n_i = '0' then
        i2c_divider <= (others => '0');
        i2c_tick    <= '0';
      else
        if(i2c_divider = unsigned(regs_out.cr_clk_div_o)) then
          i2c_tick <= '1';
          i2c_divider <= (others => '0');
        else
          i2c_tick <= '0';
          i2c_divider <= i2c_divider + 1;
        end if;
      end if;
    end if;
  end process;

  p_i2c_fsm : process(clk_sys_i)
    variable i2c_last : boolean;
  begin
    if rising_edge(clk_sys_i) then
      if rst_n_i = '0' then
        state       <= IDLE;
        seq_count   <= (others => '0');
        scl_out_fsm <= '1';
        sda_out_fsm <= '1';
      else
        case state is
          when IDLE =>
            if(regs_out.cr_load_o = '1') then
              state <= SI_START0;
              fbdiv_current <= fbdiv_base;
            elsif(fbdiv_new_p = '1') then
              state <= SI_START7;
              fdiv_adj_scaled_current <= fdiv_adj_scaled;
            end if;
          
          --https://www.silabs.com/documents/public/data-sheets/si549-datasheet.pdf
          --Set page register to point to page 0
          when SI_START0 =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", START, scl_out_fsm, sda_out_fsm, state, SI_ADDR0);
          when SI_ADDR0 =>
            f_i2c_iterate(i2c_tick, seq_count, regs_out.cr_i2c_addr_o, SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG255_ADDR);
          when SI_REG255_ADDR =>
            f_i2c_iterate(i2c_tick, seq_count, x"FF", SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_PAGE);
          when SI_PAGE =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_STOP0);
          when SI_STOP0 =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", STOP, scl_out_fsm, sda_out_fsm, state, SI_START1);

          -- Disable FCAL override (to allow FCAL for this Freq Update)
          when SI_START1 =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", START, scl_out_fsm, sda_out_fsm, state, SI_ADDR1);
          when SI_ADDR1 =>
            f_i2c_iterate(i2c_tick, seq_count, regs_out.cr_i2c_addr_o, SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG69_ADDR);
          when SI_REG69_ADDR =>
            f_i2c_iterate(i2c_tick, seq_count, x"45", SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_DIS_FCAL);
          when SI_DIS_FCAL =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_STOP1);
          when SI_STOP1 =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", STOP, scl_out_fsm, sda_out_fsm, state, SI_START2);

          -- Synchronously disable output
          when SI_START2 =>
          f_i2c_iterate(i2c_tick, seq_count, x"00", START, scl_out_fsm, sda_out_fsm, state, SI_ADDR2);
          when SI_ADDR2 =>
          f_i2c_iterate(i2c_tick, seq_count, regs_out.cr_i2c_addr_o, SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG17_ADDR);
          when SI_REG17_ADDR =>
          f_i2c_iterate(i2c_tick, seq_count, x"11", SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_DIS_OUTPUT);
          when SI_DIS_OUTPUT =>
          f_i2c_iterate(i2c_tick, seq_count, x"00", SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_STOP2);
          when SI_STOP2 =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", STOP, scl_out_fsm, sda_out_fsm, state, SI_START3);
          
          -- Write divider registers
          when SI_START3 =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", START, scl_out_fsm, sda_out_fsm, state, SI_ADDR3);
          when SI_ADDR3 =>
            f_i2c_iterate(i2c_tick, seq_count, regs_out.cr_i2c_addr_o, SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG23_ADDR);
          when SI_REG23_ADDR =>
            f_i2c_iterate(i2c_tick, seq_count, x"17", SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG23);
          when SI_REG23 =>
            f_i2c_iterate(i2c_tick, seq_count, hsdiv(7 downto 0), SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG24);
          when SI_REG24 =>
            f_i2c_iterate(i2c_tick, seq_count, b"00" & lsdiv(2 downto 0) & hsdiv(10 downto 8), SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_STOP3);
          when SI_STOP3 =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", STOP, scl_out_fsm, sda_out_fsm, state, SI_START4);
          
          -- Write divider registers
          when SI_START4 =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", START, scl_out_fsm, sda_out_fsm, state, SI_ADDR4);
          when SI_ADDR4 =>
            f_i2c_iterate(i2c_tick, seq_count, regs_out.cr_i2c_addr_o, SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG26_ADDR);
          when SI_REG26_ADDR =>
            f_i2c_iterate(i2c_tick, seq_count, x"1A", SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG26);
          when SI_REG26 =>
            f_i2c_iterate(i2c_tick, seq_count, std_logic_vector(fbdiv_current(7 downto 0)), SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG27);
          when SI_REG27 =>
            f_i2c_iterate(i2c_tick, seq_count, std_logic_vector(fbdiv_current(15 downto 8)), SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG28);
          when SI_REG28 =>
            f_i2c_iterate(i2c_tick, seq_count, std_logic_vector(fbdiv_current(23 downto 16)), SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG29);
          when SI_REG29 =>
            f_i2c_iterate(i2c_tick, seq_count, std_logic_vector(fbdiv_current(31 downto 24)), SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG30);
          when SI_REG30 =>
            f_i2c_iterate(i2c_tick, seq_count, std_logic_vector(fbdiv_current(39 downto 32)), SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG31);
          when SI_REG31 =>
            f_i2c_iterate(i2c_tick, seq_count, "00000" & std_logic_vector(fbdiv_current(42 downto 40)), SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_STOP4);
          when SI_STOP4 =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", STOP, scl_out_fsm, sda_out_fsm, state, SI_START5);

          -- Start FCAL using new divider values
          when SI_START5 =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", START, scl_out_fsm, sda_out_fsm, state, SI_ADDR5);
          when SI_ADDR5 =>
            f_i2c_iterate(i2c_tick, seq_count, regs_out.cr_i2c_addr_o, SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG07_ADDR);
          when SI_REG07_ADDR =>
            f_i2c_iterate(i2c_tick, seq_count, x"07", SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_START_FCAL);
          when SI_START_FCAL =>
            f_i2c_iterate(i2c_tick, seq_count, x"08", SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_STOP5);
          when SI_STOP5 =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", STOP, scl_out_fsm, sda_out_fsm, state, SI_START6);

          -- Synchonously enable output 
          when SI_START6 =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", START, scl_out_fsm, sda_out_fsm, state, SI_ADDR6);
          when SI_ADDR6 =>
            f_i2c_iterate(i2c_tick, seq_count, regs_out.cr_i2c_addr_o, SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG17_ADDR_2);
          when SI_REG17_ADDR_2 =>
            f_i2c_iterate(i2c_tick, seq_count, x"11", SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_EN_OUTPUT);
          when SI_EN_OUTPUT =>
            f_i2c_iterate(i2c_tick, seq_count, x"01", SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_STOP6);
          when SI_STOP6 =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", STOP, scl_out_fsm, sda_out_fsm, state, IDLE);



          when SI_START7 =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", START, scl_out_fsm, sda_out_fsm, state, SI_ADDR7);
          when SI_ADDR7 =>
            f_i2c_iterate(i2c_tick, seq_count, regs_out.cr_i2c_addr_o, SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_REG231_ADDR);
          when SI_REG231_ADDR =>
            f_i2c_iterate(i2c_tick, seq_count, x"E7", SEND_BYTE, scl_out_fsm, sda_out_fsm, state, ADPLL_DELTA_M_0);
          when ADPLL_DELTA_M_0 =>
            f_i2c_iterate(i2c_tick, seq_count, std_logic_vector(fdiv_adj_scaled_current(7 downto 0)), SEND_BYTE, scl_out_fsm, sda_out_fsm, state, ADPLL_DELTA_M_1);
          when ADPLL_DELTA_M_1 =>
            f_i2c_iterate(i2c_tick, seq_count, std_logic_vector(fdiv_adj_scaled_current(15 downto 8)), SEND_BYTE, scl_out_fsm, sda_out_fsm, state, ADPLL_DELTA_M_2);
          when ADPLL_DELTA_M_2 =>
            f_i2c_iterate(i2c_tick, seq_count, std_logic_vector(fdiv_adj_scaled_current(23 downto 16)), SEND_BYTE, scl_out_fsm, sda_out_fsm, state, SI_STOP7);
          when SI_STOP7 =>
            f_i2c_iterate(i2c_tick, seq_count, x"00", STOP, scl_out_fsm, sda_out_fsm, state, IDLE);
            
          when others => null;
        end case;
      end if;
    end if;
  end process;

  regs_in.gpsr_scl_i <= scl_pad_i;
  regs_in.gpsr_sda_i <= sda_pad_i;

  p_host_i2c : process(clk_sys_i)
  begin
    if rising_edge(clk_sys_i) then
      if(rst_n_i = '0') then
        scl_out_host <= '1';
        sda_out_host <= '1';
      else
        if(regs_out.gpsr_scl_load_o = '1' and regs_out.gpsr_scl_o = '1') then
          scl_out_host <= '1';
        end if;

        if(regs_out.gpsr_sda_load_o = '1' and regs_out.gpsr_sda_o = '1') then
          sda_out_host <= '1';
        end if;

        if(regs_out.gpcr_scl_o = '1') then
          scl_out_host <= '0';
        end if;

        if(regs_out.gpcr_sda_o = '1') then
          sda_out_host <= '0';
        end if;
        
      end if;
    end if;
  end process;

  p_mux_i2c : process(clk_sys_i)
  begin
    if rising_edge(clk_sys_i) then
      if(state = IDLE) then
        scl_pad_oen_o <= scl_out_host;
        sda_pad_oen_o <= sda_out_host;
      else
        scl_pad_oen_o <= scl_out_fsm;
        sda_pad_oen_o <= sda_out_fsm;
      end if;
    end if;
  end process;

  wb_err_o <= '0';
  
end rtl;
