
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;

use work.gencores_pkg.all;
use work.wrcore_pkg.all;
use work.wr_fabric_pkg.all;

library UNISIM;
use UNISIM.vcomponents.all;

library work;
use work.wishbone_pkg.all;

entity zcu102 is
  generic
    (
      TAR_ADDR_WDTH : integer := 13     -- not used for this project
      );
  port
    (
      --Clocks
      clk_25m_vcxo_n_i : in std_logic;  -- 25MHz VCXO clock
      clk_25m_vcxo_p_i : in std_logic;  -- 25MHz VCXO clock

      gtp_dedicated_clk_p_i : in std_logic;  -- Dedicated clock for Xilinx GTP transceiver
      gtp_dedicated_clk_n_i : in std_logic;

      --Dedicated CLK to configure the AD9516 PLL (125MHz by default)
      clk_125mhz_p_i : in std_logic;
      clk_125mhz_n_i : in std_logic;

      -- Secundary ref clock.
      clk_sec_ref_p_i : in std_logic;
      clk_sec_ref_n_i : in std_logic;
      
      clk_sys_i : in std_logic;
      
      clk_25m_vcxo_bufg_o : out std_logic;
      
      clk_dmtd_o : out std_logic;

      clk_rx_o : out std_logic;

      -- DAC Signals
      dac_sclk_o  : out std_logic;
      dac_din_o   : out std_logic;
--      dac_clr_n_o : out std_logic;
      dac_cs1_n_o : out std_logic;
      dac_cs2_n_o : out std_logic;

      fpga_scl_b : inout std_logic;
      fpga_sda_b : inout std_logic;
      
      si549_scl_b : inout std_logic;
      si549_sda_b : inout std_logic;

      button_rst_i : in std_logic := '0';
      button2_i    : in std_logic := '0';

      thermo_id : inout std_logic;      -- 1-Wire interface for DS18B20

      -------------------------------------------------------------------------
      -- GTP0 pins
      -------------------------------------------------------------------------
      gtp0_txp_o : out std_logic;
      gtp0_txn_o : out std_logic;

      gtp0_rxp_i : in std_logic;
      gtp0_rxn_i : in std_logic;

      gtp0_mod_def0_b    : in    std_logic;  -- gtp0 detect
      gtp0_mod_def1_b    : inout std_logic;  -- scl
      gtp0_mod_def2_b    : inout std_logic;  -- sda
      gtp0_rate_select_b : inout std_logic;
      gtp0_tx_fault_i    : in    std_logic;
      gtp0_tx_disable_o  : out   std_logic;

      -----------------------------------------
      --UART
      -----------------------------------------
      uart_rxd_i : in  std_logic;
      uart_txd_o : out std_logic;
      uart_rts_o : out std_logic;
      uart_cts_i : in  std_logic;

      ------------------------------------------
      -- EXT CLK REF (GM)
      ------------------------------------------
      ext_clk_10m_i         : in std_logic;
      ext_clk_625m_i        : in std_logic;
      ext_625m_clk_locked   : in std_logic;

      --PPS in
      pps_i : in std_logic;

      --PPS out
      pps_out_o   : out std_logic;

      --phy_rst_n_o     : out std_logic := '0'

      -- I2C
      len_conn_scl_b  : inout std_logic;
      len_conn_sda_b  : inout std_logic;

      -- Debugging outputs
      debug0_o : out std_logic;
      debug1_o : out std_logic;
      debug3_o : out std_logic;

      --LEDs
      led0_o    : out std_logic;
      led1_o    : out std_logic;

      -- Voltage converter enabling pin
      voltage_conv_en_o : out std_logic;
           
      clk_125mhz_o      : out std_logic;

          debug_o    : out std_logic_vector(5 downto 0)
      );

end zcu102;

architecture rtl of zcu102 is

  component reset_gen
    port (
      clk_sys_i        : in  std_logic;
      rst_pcie_n_a_i   : in  std_logic;
      rst_button_n_a_i : in  std_logic;
      rst_n_o          : out std_logic);
  end component;

component wr_gth_phy_kintex7ultrascale
  generic (
    g_simulation : integer := 0);
  port (
      clk_gth_p_i     : in std_logic;
      clk_gth_n_i     : in std_logic;
      clk_freerun_i   : in std_logic;
      tx_out_clk_o    : out std_logic;
      tx_locked_o     : out std_logic;
      tx_data_i       : in std_logic_vector(15 downto 0);
      tx_k_i          : in std_logic_vector(1 downto 0);
      tx_disparity_o  : out std_logic;
      tx_enc_err_o    : out std_logic;
      rx_rbclk_o      : out std_logic;
      rx_data_o       : out std_logic_vector(15 downto 0);
      rx_k_o          : out std_logic_vector(1 downto 0);
      rx_enc_err_o    : out std_logic;
      rx_bitslide_o   : out std_logic_vector(4 downto 0);
      rst_i           : in std_logic;
      loopen_i        : in std_logic_vector(2 downto 0);
      debug_i         : in  std_logic_vector(15 downto 0);
      debug_o         : out std_logic_vector(15 downto 0);
      pad_txn_o       : out std_logic;
      pad_txp_o       : out std_logic;
      pad_rxn_i       : in std_logic := '0';
      pad_rxp_i       : in std_logic := '0';
      rdy_o           : out std_logic);
  end component;
  ------------------------------------------------------------------------------
  -- Constants declaration
  ------------------------------------------------------------------------------
  constant c_NUM_PHYS          : integer := 2;

  -- Wishbone constant
  constant c_interface_mode       : t_wishbone_interface_mode       := PIPELINED;
  constant c_address_granularity  : t_wishbone_address_granularity  := BYTE;

  -- GPIO
  constant c_NUM_GPIO_PINS          : integer := 2;

  constant c_GPIO_CONN_SCL          : integer := 0; -- I2C LEN connector bus.
  constant c_GPIO_CONN_SDA          : integer := 1; -- I2C LEN connector bus.

  ------------------------------------------------------------------------------
  -- Signals declaration
  ------------------------------------------------------------------------------
  signal s_txuart : std_logic;
  signal s_rxuart : std_logic;

  -- Reference clock disciplined by WR. 62.5 MHz
  signal wr_ref_clk : std_logic;

  signal pllout_clk_sys         : std_logic;
  signal pllout_clk_dmtd        : std_logic;
  signal pllout_clk_fb_dmtd     : std_logic;

  signal clk_25m_vcxo_buf : std_logic;
  signal clk_25m_vcxo_bufg : std_logic;
  signal clk_sys          : std_logic;
  signal clk_dmtd         : std_logic;
  signal dac_rst_n        : std_logic;

  signal wrc_scl_o : std_logic;
  signal wrc_scl_i : std_logic;
  signal wrc_sda_o : std_logic;
  signal wrc_sda_i : std_logic;



  signal gtp0_scl_o : std_logic;
  signal gtp0_scl_i : std_logic;
  signal gtp0_sda_o : std_logic;
  signal gtp0_sda_i : std_logic;
  signal gtp1_scl_o : std_logic;
  signal gtp1_scl_i : std_logic;
  signal gtp1_sda_o : std_logic;
  signal gtp1_sda_i : std_logic;
  
  signal si549_scl_i : std_logic;
  signal si549_scl_o : std_logic;
  signal si549_sda_i : std_logic;
  signal si549_sda_o : std_logic;

  signal dac_hpll_load_p1 : std_logic;
  signal dac_dpll_load_p1 : std_logic;
  signal dac_hpll_data    : std_logic_vector(15 downto 0);
  signal dac_dpll_data    : std_logic_vector(15 downto 0);

  signal local_reset_n     : std_logic;
  signal local_reset_n_d0  : std_logic;
  signal local_reset_n_d1  : std_logic;

  --signal button1_synced : std_logic_vector(2 downto 0);
  signal owr_en : std_logic_vector(1 downto 0);
  signal owr_i  : std_logic_vector(1 downto 0);

  --Duplicate GTPs. Duplicate signals from/to GTPs.
  --WR_LEN.
	signal to_phys   : t_phyif_output_array(c_NUM_PHYS-1 downto 0);
	signal from_phys : t_phyif_input_array(c_NUM_PHYS-1 downto 0);

  -- 125 MHz clock to configure the AD9516
  signal s_125mhz_clk : std_logic;
  signal s_125mhz_clkg : std_logic;

  -- 62.5 MHz clock from the AD9516
  signal s_sec_ref_clk  : std_logic;
  signal s_sec_ref_clkg : std_logic;

	--- PLL external signals
  signal s_PLL_en       : std_logic;
  signal s_PLL_done     : std_logic;
  signal s_PLL_ERR      : std_logic;
  signal rst_PLLor_err  : std_logic;

  signal s_pll_sync_n     : std_logic;

  --External 10 MHz clock
  signal ext_clk        : std_logic;

  --For debugging the input PPS
  signal s_pps_input         : std_logic;

  --Set Grand Master mode. It enables the PPS input.
  signal set_gm          : std_logic;

  -- Signals to latch the PPS output with the WR clk.
  signal pps        : std_logic;
  signal pps_out    : std_logic;

  --Signals to manage the external 10 MHz clock
  signal s_ext_10m_clk              : std_logic;
  signal s_ext_10m_clkg             : std_logic;
  signal s_ext_625m_clk             : std_logic;
  signal s_ext_625m_clkg            : std_logic;
  signal s_clk_ext_mul          : std_logic := '0';
  signal s_ext_625m_clk_locked   : std_logic := '0';

  -------------------------------------------------------------
  --SYNC stuff
  signal s_pps_inter            :	std_logic;
  signal s_pps_inter_pre        :	std_logic;
  signal s_pps_valid_inter      :	std_logic;
  signal s_pps_valid_inter_pre  :	std_logic;
  signal s_sync_cnt             : std_logic_vector(3 downto 0) := (others => '0');
  signal s_sync                 : std_logic;


begin

  -- PLL stuff
  pllout_clk_dmtd <= clk_25m_vcxo_bufg;

  -- The system is reset when the button is pressed.
  n0: process (clk_sys, button_rst_i) is		-- EASE/HDL sens.list
  begin
    if(rising_edge(clk_sys)) then
      local_reset_n_d0 <=  (not button_rst_i);
      local_reset_n_d1 <=  local_reset_n_d0;
      local_reset_n    <=  local_reset_n_d1;
    end if;
  end process n0;

  clk_sys <= clk_sys_i;

  cmp_clk_dmtd_buf : BUFG
    port map (
      O => clk_dmtd,
      I => pllout_clk_dmtd);
      
  clk_dmtd_o <= clk_dmtd;    

  cmp_25mhz_clk_buf : IBUFDS
    generic map (
      DQS_BIAS => "FALSE")
    port map (
      O  => clk_25m_vcxo_buf, -- Buffer output
      I  => clk_25m_vcxo_p_i, -- Diff_p buffer input (connect directly to top-level port)
      IB => clk_25m_vcxo_n_i  -- Diff_n buffer input (connect directly to top-level port)
      );

  cmp_25mhz_bufg : BUFGCE
    port map (
      CE  => '1',
      O   => clk_25m_vcxo_bufg,
      I   => clk_25m_vcxo_buf);

  -----------------------------------------------------------------------------
  -- The infamous registering process to ensure the PPS & 10 MHz stability strikes again.
  -----------------------------------------------------------------------------
    p_latch_pps : process(wr_ref_clk)
    begin
      if rising_edge(wr_ref_clk) then
       pps_out  <= pps;
      end if;
    end process;

  debug0_o  <= wr_ref_clk;
  debug1_o  <= pps_out;
  --debug3_o  <= ;

    pps_out_o <= pps_out;

	------------------------------------------------------------------------------
  -- 125 MHz clock to initialize the AD9516
  ------------------------------------------------------------------------------
  cmp_125mhz_clk : IBUFDS
  generic map (
    DQS_BIAS => "FALSE")
  port map (
      O  => s_125mhz_clk,
      I  => clk_125mhz_p_i,
      IB => clk_125mhz_n_i);

  cmp_125mhz_clkg : BUFGCE
    port map (
      CE  => '1',
      O   => s_125mhz_clkg,
      I   => s_125mhz_clk);


  ------------------------------------------------------------------------------
  -- 62.5 clk secundary clock from the AD9516
  ------------------------------------------------------------------------------
  cmp_sec_ref_clk : IBUFDS
  generic map (
    DQS_BIAS => "FALSE")
  port map (
      O  => s_sec_ref_clk,
      I  => clk_sec_ref_p_i,
      IB => clk_sec_ref_n_i);

  cmp_sec_ref_clkg : BUFGCE
    port map (
      CE  => '1',
      O   => s_sec_ref_clkg,
      I   => s_sec_ref_clk);

  ------------------------------------------------------------------------------
  -- External MHz clock
  ------------------------------------------------------------------------------
  ext_clk_10m_buf : IBUFG
    port map (
      O => s_ext_10m_clk,
      I => ext_clk_10m_i);
  
  ext_clk_625m_buf : IBUFG
    port map (
      O => s_ext_625m_clk,
      I => ext_clk_625m_i);

  -- The s_ext_clk is buffered as global clock again to avoid timing constraint violations.
  cmp_ext_10m_clk : BUFG
    port map(
      O => s_ext_10m_clkg,
      I => s_ext_10m_clk);
      
  cmp_ext_625m_clk : BUFG
    port map(
      O => s_ext_625m_clkg,
      I => s_ext_625m_clk);

------------------------------------------------------------------------------

  -- EEPROM
  fpga_scl_b <= '0' when wrc_scl_o = '0' else 'Z';
  fpga_sda_b <= '0' when wrc_sda_o = '0' else 'Z';
  wrc_scl_i  <= fpga_scl_b;
  wrc_sda_i  <= fpga_sda_b;

  --SFP 1 I2C
  gtp0_mod_def1_b <= '0' when gtp0_scl_o = '0' else 'Z';
  gtp0_mod_def2_b <= '0' when gtp0_sda_o = '0' else 'Z';
  gtp0_scl_i      <= gtp0_mod_def1_b;
  gtp0_sda_i      <= gtp0_mod_def2_b;
  
  -- Si549
  si549_scl_b <= '0' when si549_scl_o = '0' else 'Z';
  si549_sda_b <= '0' when si549_sda_o = '0' else 'Z';
  si549_scl_i <= si549_scl_b;
  si549_sda_i <= si549_sda_b;


  thermo_id <= '0' when owr_en(0) = '1' else 'Z';
  owr_i(0)  <= thermo_id;
  
  s_ext_625m_clk_locked <= ext_625m_clk_locked;

  U_WR_CORE : xwr_core
    generic map (
      g_simulation                => 0,
      g_with_external_clock_input => true,
      g_phys_uart                 => true,
      g_virtual_uart              => true,
      g_aux_clks                  => 0,
      g_ep_rxbuf_size             => 1024,
      g_pcs_16bit                 => true,                 
      g_dpram_initf               => "",
      g_dpram_size                => 131072/4,
      g_interface_mode            => c_interface_mode,
      g_address_granularity       => c_address_granularity,
      g_aux_sdb                   => c_wrc_periph3_sdb,
      g_softpll_enable_debugger   => false,
      g_vuart_fifo_size           => 1024)
    port map (
      clk_sys_i             => clk_sys,
      clk_dmtd_i            => clk_dmtd,
      clk_ref_i             => wr_ref_clk,
      clk_aux_i             => (others => '0'),
      clk_ext_i             => s_ext_10m_clkg,
      clk_ext_mul_i         => s_ext_625m_clkg,
      clk_ext_mul_locked_i  => s_ext_625m_clk_locked,
      pps_ext_i             => s_pps_input,
      rst_n_i               => local_reset_n,
      dac_hpll_load_p1_o    => dac_hpll_load_p1,
      dac_hpll_data_o       => dac_hpll_data,
      dac_dpll_load_p1_o    => dac_dpll_load_p1,
      dac_dpll_data_o       => dac_dpll_data,
      phy_ref_clk_i         => wr_ref_clk,
      phy_rst_o            => to_phys(0).rst,
      phy_loopen_o         => open,
      phy_loopen_vec_o     => to_phys(0).loopen,
      phy_tx_data_o        => to_phys(0).tx_data,
      phy_tx_k_o           => to_phys(0).tx_k,
      phy_sfp_tx_disable_o => to_phys(0).tx_disable,
      phy_tx_prbs_sel_o    => to_phys(0).tx_prbs_sel,

      phy_tx_disparity_i   => from_phys(0).tx_disparity,
      phy_tx_enc_err_i     => from_phys(0).tx_enc_err,
      phy_rx_data_i        => from_phys(0).rx_data,
      phy_rx_rbclk_i       => from_phys(0).rx_clk,
      phy_rx_k_i           => from_phys(0).rx_k,
      phy_rx_enc_err_i     => from_phys(0).rx_enc_err,
      phy_rx_bitslide_i    => from_phys(0).rx_bitslide,
      phy_rdy_i            => from_phys(0).rdy,
      phy_sfp_tx_fault_i   => from_phys(0).tx_fault,
      phy_sfp_los_i        => from_phys(0).los,
      led_act_o            => led1_o,
      led_link_o           => led0_o,
      scl_o                => wrc_scl_o,
      scl_i                => wrc_scl_i,
      sda_o                => wrc_sda_o,
      sda_i                => wrc_sda_i,
      --I2C for SFP1
      sfp_scl_o            => gtp0_scl_o,
      sfp_scl_i            => gtp0_scl_i,
      sfp_sda_o            => gtp0_sda_o,
      sfp_sda_i            => gtp0_sda_i,
      si549_scl_o          => si549_scl_o,
      si549_scl_i          => si549_scl_i,
      si549_sda_o          => si549_sda_o,
      si549_sda_i          => si549_sda_i,
      sfp_det_i            => gtp0_mod_def0_b,
      btn1_i               => (not button_rst_i),
      btn2_i               => '1', 
      spi_sclk_o           => open,
      spi_ncs_o            => open,
      spi_mosi_o           => open,
      spi_miso_i           => '0',
      uart_rxd_i           => s_rxuart,
      uart_txd_o           => s_txuart,
      owr_pwren_o          => open,
      owr_en_o             => owr_en,
      owr_i                => owr_i,
      slave_i              => cc_dummy_slave_in,
      slave_o              => open,
      aux_master_o         => open,
      aux_master_i         => cc_dummy_master_in,
      wrf_src_o            => open,
      wrf_src_i            => c_dummy_src_in,
      wrf_snk_o            => open,
      wrf_snk_i            => c_dummy_snk_in,
      timestamps_o         => open,
      timestamps_ack_i     => '1',
      fc_tx_pause_req_i    => '0',
      fc_tx_pause_delay_i  => x"0000",
      fc_tx_pause_ready_o  => open,
      tm_link_up_o         => open,
      tm_dac_value_o       => open,
      tm_dac_wr_o          => open,
      tm_clk_aux_lock_en_i => (others => '0'),
      tm_clk_aux_locked_o  => open,
      tm_time_valid_o      => open,
      tm_tai_o             => open,
      tm_cycles_o          => open,
      pps_p_o              => s_pps_inter,
      pps_valid_o          => s_pps_valid_inter,
      pps_led_o            => open,
      rst_aux_n_o          => open,
      link_ok_o            => open,
      debug_o              => debug_o);

    clk_rx_o <= from_phys(0).rx_clk;

    -- UART connections.
    s_rxuart   <= uart_rxd_i;
    uart_txd_o <= s_txuart;
    uart_rts_o <= '1';

   pps  <= s_pps_inter;

       -- GTH Instance.

  U_GTH_instance : wr_gth_phy_kintex7ultrascale
    generic map (
      g_simulation => 0)
    port map (
        clk_gth_p_i     => gtp_dedicated_clk_p_i,
        clk_gth_n_i     => gtp_dedicated_clk_n_i,
        rst_i           => not local_reset_n,

        clk_freerun_i   => clk_sys, -- This clock must be 62.5. Is it the DMTD clock good enough?
        tx_locked_o     => open,

        tx_out_clk_o    => wr_ref_clk,
        tx_data_i       => to_phys(0).tx_data,
        tx_k_i          => to_phys(0).tx_k,
        tx_disparity_o  => from_phys(0).tx_disparity,
        tx_enc_err_o    => from_phys(0).tx_enc_err,
        rx_rbclk_o      => from_phys(0).rx_clk,
        rx_data_o       => from_phys(0).rx_data,
        rx_k_o          => from_phys(0).rx_k,
        rx_enc_err_o    => from_phys(0).rx_enc_err,
        rx_bitslide_o   => from_phys(0).rx_bitslide,
        rdy_o           => from_phys(0).rdy,
        loopen_i        => to_phys(0).loopen,

        debug_i         => (others => '0'),
        debug_o         => open,

        pad_txn_o       => gtp0_txn_o,
        pad_txp_o       => gtp0_txp_o,
        pad_rxn_i       => gtp0_rxn_i,
        pad_rxp_i       => gtp0_rxp_i);

	---------------------------------------------------------------

  U_DAC_ARB : spec_serial_dac_arb
    generic map (
      g_invert_sclk    => false,
      g_num_extra_bits => 8)

    port map (
      clk_i   => clk_sys,
      rst_n_i => local_reset_n,

      val1_i  => dac_dpll_data,
      load1_i => dac_dpll_load_p1,

      val2_i  => dac_hpll_data,
      load2_i => dac_hpll_load_p1,

      dac_cs_n_o(0) => dac_cs1_n_o,
      dac_cs_n_o(1) => dac_cs2_n_o,
      dac_clr_n_o   => open,
      dac_sclk_o    => dac_sclk_o,
      dac_din_o     => dac_din_o);

  -- From/To the external SFP cages.
  from_phys(0).tx_fault <= gtp0_tx_fault_i;
  gtp0_tx_disable_o     <= '1';


  s_pps_input <= pps_i;
 -- Tied to VCC
  voltage_conv_en_o <= '1';


  clk_25m_vcxo_bufg_o <= clk_25m_vcxo_bufg;

  clk_125mhz_o <= s_125mhz_clkg;

end rtl;
