#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <wrc.h>

#include "si549.h"

#include "hw/si549_if_wb.h"

/*
https://www.silabs.com/documents/public/data-sheets/si549-datasheet.pdf

To configure the si549 to 25 MHz:
1:Get the minimum (HSDIV*LSDIV). Minimum (HSDIV*LSDIV) = (11.5000 GHz / 62.4961 MHz) = 184.0114 -> 186
2:Find LSDIV. The HSDIV maximum divider is 2046. 184 is less than 2046, so the LSDIV divide ratio is 1, therefore LSDIV register value = 0
3:Find HSDIV. The HSDIV value must be: 5-33 even or odd values or 34-2046 even values only. The 184 value is a valid value.
4:Calculate a valid VCO frequency. 62.4961MHz * 186(HSDIV) = 11.6242746 GHz. This VCO frequency is into the valid range.
5:Calculate FDIV value. 11.6242746 GHz (VCO) / 152.6 MHz (Crystal ref) = 76.174800786
6:The FBINT value is the integer part of FDIV: 76
7:Multiply the fractional part of FBINT by 2^32: 0.174800786 * 2^32 = 750763659,185094656. The integer part of the result is FBFRAC
*/

/*
https://www.silabs.com/documents/public/data-sheets/si549-datasheet.pdf

To configure the si549 to 25 MHz:
1:Get the minimum (HSDIV*LSDIV). Minimum (HSDIV*LSDIV) = (11.5000 GHz / 62,5 MHz) = 184
2:Find LSDIV. The HSDIV maximum divider is 2046. 184 is less than 2046, so the LSDIV divide ratio is 1, therefore LSDIV register value = 0
3:Find HSDIV. The HSDIV value must be: 5-33 even or odd values or 34-2046 even values only. The 184 value is a valid value.
4:Calculate a valid VCO frequency. 62.5MHz * 184(HSDIV) = 11.500 GHz. This VCO frequency is into the valid range.
5:Calculate FDIV value. 11.500 GHz (VCO) / 152.6 MHz (Crystal ref) = 75.360419397
6:The FBINT value is the integer part of FDIV: 75
7:Multiply the fractional part of FBINT by 2^32: 0.360419397 * 2^32 = 1547989522,959040512. The integer part of the result is FBFRAC
*/

#define LSDIV  0x00
//#define HSDIV  0xB8        //184
#define FBINT  0x4C         //76
#define FBFRAC 0x2CBFBE8B   //750763659  
//#define I2C_ADDR 0x67
#define I2C_ADDR 0xCE
#define CLK_DIV 0xA0       // This is 625. 62.5 MHz / 625 = 100 KHz

#define si549_write(reg, val) \
	*(volatile uint32_t *) (BASE_SI549_IFACE + reg) = (val)

#define si549_read(reg) \
	*(volatile uint32_t *) (BASE_SI549_IFACE + reg)

uint8_t gain = 50;
uint8_t hsdiv = 0xBA;

void si549_initalize()
{
    uint32_t temp = ((I2C_ADDR & SI549_CR_I2C_ADDR_MASK) | 
                                ((CLK_DIV) << SI549_CR_CLK_DIV_SHIFT) | 
                                SI549_CR_ENABLE |
                                gain << SI549_CR_GAIN_SHIFT);

    pp_printf("si549_initalize 0x%08x\n", temp);                            
    //Set the control register
    si549_write(SI549_REG_CR, temp);

    si549_write(SI549_REG_HSDIV, hsdiv);
    si549_write(SI549_REG_LSDIV, LSDIV);
    si549_write(SI549_REG_FDIV_FBFRAC, FBFRAC);
    si549_write(SI549_REG_FDIV_FBINT, FBINT);

    temp |= SI549_CR_LOAD;

    si549_write(SI549_REG_CR, temp);
}

void si549_wait_until_ready()
{
    uint32_t value;
    
    value = si549_read(SI549_REG_CR);
    //pp_printf("si549_wait_until_ready 0x%08x\n", value);
    while((value & SI549_CR_BUSY) != 0)
    {
        value = si549_read(SI549_REG_CR);
        //pp_printf("si549_wait_until_ready 0x%08x\n", value);
    }

}

void si549_set_gain(uint8_t input)
{
    gain = input;

    uint32_t temp = ((I2C_ADDR & SI549_CR_I2C_ADDR_MASK) | 
                                ((CLK_DIV) << SI549_CR_CLK_DIV_SHIFT) | 
                                SI549_CR_ENABLE |
                                gain << SI549_CR_GAIN_SHIFT);

    pp_printf("si549_set_gain 0x%08x\n", temp);                            
    //Set the control register
    si549_write(SI549_REG_CR, temp);
}

uint8_t si549_get_gain()
{
    return gain;
}

void si549_set_hsdiv(uint8_t value)
{
    hsdiv = value;
    si549_initalize();
}

uint8_t si549_get_hsdiv()
{
    return hsdiv;
}