/*
 * fpgabgd core library:
 *
 * Access to the FPGA wishbone bus using the FPGABGD mapped device.
 *
 *  Created on: 09.09.2020
 *  Authors:
 * 		- Pilar Gil/Manuel Pena/Adrian Martinez
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License...
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdbool.h>

#include <unistd.h>
#include <sys/mman.h>
#include <inttypes.h>
#include <fcntl.h>
#include <poll.h>
#include <string.h>

#include "axi.h"

bool 
FPGABGD_isValid( FPGABGD_DEV_HANDLER hDev ) 
{
	return hDev!=NULL;
}

FPGABGD_DEV_HANDLER 
FPGABGD_open( uint64_t base, uint32_t size )
{
	FPGABGD_DEV_HANDLER hDev = 0;
	int fd;
	void *_fpga_base_virt;

	fd = open( "/dev/mem", O_RDWR | O_SYNC );
	if( fd < 0 ){
		fprintf( stderr,
			 "Error while opening device: %s\n Error: %s",
			 "/dev/mem",
			 strerror( errno ));
		hDev->fd = fd;		
		return hDev;
	}

	_fpga_base_virt = mmap( NULL, size, ( PROT_WRITE | PROT_READ | MAP_PRIVATE ), MAP_SHARED, fd, base );

	if( _fpga_base_virt == MAP_FAILED )
	{
		int lerrno = errno;
		fprintf( stderr,
			 "Map failed: Error: %s\n",
			 strerror( lerrno ));		
		close( fd );		
		hDev->fd = -1;
		return hDev;
	}

	hDev = (FPGABGD_DEV_HANDLER)malloc(sizeof( FPGABGD_DEV_STRUCT ));
	set_filedescriptor( hDev, fd );
	hDev->fpga_base_virt = _fpga_base_virt;
	hDev->size = size;	
	hDev->physical_buffer_address = base;
	return hDev;
}


void 
set_filedescriptor( FPGABGD_DEV_HANDLER hDev, int const _fd ){
	hDev->fd = _fd;	
}

int 
get_filedescriptor( FPGABGD_DEV_HANDLER hDev ){
	return hDev->fd;
}

void 
FPGABGD_close(FPGABGD_DEV_HANDLER hDev)
{

	if (munmap((void*)hDev->fpga_base_virt, hDev->size ) == -1)
		printf("Unmapping failed \n");
	
	hDev->fpga_base_virt = 0;
	close( hDev->fd );
	free( hDev );
}

uint32_t 
FPGABGD_WB_read_32( FPGABGD_DEV_HANDLER hDev, uint32_t reg )
{
	return (*(volatile uint32_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address) )));
}

void 
FPGABGD_WB_write_32(FPGABGD_DEV_HANDLER hDev,uint32_t reg, uint32_t val)
{
	*((volatile uint32_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address)  ))) = val;
}

uint16_t 
FPGABGD_WB_read_16( FPGABGD_DEV_HANDLER hDev, uint32_t reg )
{
	return (*(volatile uint16_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address) )));
}

void 
FPGABGD_WB_write_16( FPGABGD_DEV_HANDLER hDev, uint32_t reg, uint16_t val )
{
	*((volatile uint16_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address)  ))) = val;
}

uint8_t 
FPGABGD_WB_read_8( FPGABGD_DEV_HANDLER hDev, uint32_t reg )
{
	return (*(volatile uint8_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address) )));
}

void 
FPGABGD_WB_write_8( FPGABGD_DEV_HANDLER hDev,uint32_t reg, uint8_t val )
{
	*((volatile uint8_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address)))) = val;
}

uint32_t 
FPGABGD_WB_read_bit( FPGABGD_DEV_HANDLER hDev, uint32_t reg, int bit_pos )
{
	uint32_t memory_read = (*(volatile uint32_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address))));
	return(memory_read >> bit_pos) & 1U;
}

void 
FPGABGD_WB_write_bit( FPGABGD_DEV_HANDLER hDev, uint32_t reg, int bit_pos, int bit_val )
{
	uint32_t new_value = ((*(volatile uint32_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address)))) & ~(1 << bit_pos)) | (bit_val << bit_pos);
	(*(volatile uint32_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address)))) = new_value;
}


int 
FPGABGD_wishbone_RW_32( FPGABGD_DEV_HANDLER hDev, uint32_t addr, uint32_t *pData, int fToDev )
{
	int status=0;

	if(fToDev == 1)
	{
		FPGABGD_WB_write_32(hDev,addr,pData[0]);
		status = 1; //OK
	}else{
		pData[0] = FPGABGD_WB_read_32( hDev, addr );
		status = 1; //OK
	}
	return status;
}


void
FPGABGD_print_block( uint64_t base, uint32_t offset, uint32_t size )
{
	FPGABGD_DEV_HANDLER hDev = FPGABGD_open( base, size );
	uint32_t data = 0;
	
	for ( uint32_t index = 0; index <= size; index += 4)
	{
		data = FPGABGD_WB_read_32(hDev,  (base + offset) + index);
		printf("Value =  0x%x \n", data );
	}
	FPGABGD_close( hDev );
}

uint32_t *
FPGABGD_get_buffer( FPGABGD_DEV_HANDLER hDev, uint32_t offset , uint32_t nsize, bool to_dev )
{
	return (uint32_t*)(hDev->fpga_base_virt  + offset);
}
