/**
 * @file PLL_hmc7044_programmer
 *
 * @brief Program to configure PLL device
 * @author Adrian Martinez Munera adrian.martinez@sevensols.com
 * @ingroup LLRF SARAF
 * @date 04 November 2020
 * @copyright Copyright (c) 2020 Seven Solutions S.L (www.sevensols.com)
 * 
 * DEBUG MODE --> run --> "./PLL_hmc7044_programmer.c 1"
 *
 * This file is part of PROYECT
 * You might use, distribute and modify this code and its resulting
 * binary form under the terms of the LICENSE.txt provided within the
 * module/project: PROYECT.
 *
 * If you do not have received a copy of the LICENSE.txt along with
 * this file please write to info@sevensols.com and consider that
 * this file can not be copied and/or distributed in any forms.
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <inttypes.h>
#include <errno.h>
#include "axi.h"

#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <poll.h>
#include <stdbool.h>
#include "xspi_l.h"

#include "lmx2594_config.h"


#define PERIPH_MAPPING_SIZE   0x1000
#define CS_PLL	              0
#define dir_base_addr_qspi    0xA0020000
#define LLRF_GEN_BNK_REG3     0xC
#define LLRF_GEN_BNK_PLL_RST  3
#define LLRF_GEN_BNK_PLL_SYNC 4
#define ARRAY_SIZE(a) (sizeof(a) / sizeof((a)[0]))

#define MODE_WRITE 	0
#define MODE_READ	1
#define RESET_REGISTER	0x0
#define RESET_LMX		0x2
#define FCAL_EN			0x8
#define READ_OP			0x800000

uint32_t 
FPGABGD_WB_read_32( FPGABGD_DEV_HANDLER hDev, uint32_t reg )
{
	return (*(volatile uint32_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address) )));
}

void 
FPGABGD_WB_write_32(FPGABGD_DEV_HANDLER hDev,uint32_t reg, uint32_t val)
{
	*((volatile uint32_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address)  ))) = val;
}

uint16_t 
FPGABGD_WB_read_16( FPGABGD_DEV_HANDLER hDev, uint32_t reg )
{
	return (*(volatile uint16_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address) )));
}

void 
FPGABGD_WB_write_16( FPGABGD_DEV_HANDLER hDev, uint32_t reg, uint16_t val )
{
	*((volatile uint16_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address)  ))) = val;
}

uint8_t 
FPGABGD_WB_read_8( FPGABGD_DEV_HANDLER hDev, uint32_t reg )
{
	return (*(volatile uint8_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address) )));
}

void 
FPGABGD_WB_write_8( FPGABGD_DEV_HANDLER hDev,uint32_t reg, uint8_t val )
{
	*((volatile uint8_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address)))) = val;
}

uint32_t 
FPGABGD_WB_read_bit( FPGABGD_DEV_HANDLER hDev, uint32_t reg, int bit_pos )
{
	uint32_t memory_read = (*(volatile uint32_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address))));
	return(memory_read >> bit_pos) & 1U;
}

void 
FPGABGD_WB_write_bit( FPGABGD_DEV_HANDLER hDev, uint32_t reg, int bit_pos, int bit_val )
{
	uint32_t new_value = ((*(volatile uint32_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address)))) & ~(1 << bit_pos)) | (bit_val << bit_pos);
	(*(volatile uint32_t *)(hDev->fpga_base_virt + (reg - (hDev->physical_buffer_address)))) = new_value;
}


int 
FPGABGD_wishbone_RW_32( FPGABGD_DEV_HANDLER hDev, uint32_t addr, uint32_t *pData, int fToDev )
{
	int status=0;

	if(fToDev == 1)
	{
		FPGABGD_WB_write_32(hDev,addr,pData[0]);
		status = 1; //OK
	}else{
		pData[0] = FPGABGD_WB_read_32( hDev, addr );
		status = 1; //OK
	}
	return status;
}


void
FPGABGD_print_block( uint64_t base, uint32_t offset, uint32_t size )
{
	FPGABGD_DEV_HANDLER hDev = FPGABGD_open( base, size );
	uint32_t data = 0;
	
	for ( uint32_t index = 0; index <= size; index += 4)
	{
		data = FPGABGD_WB_read_32(hDev,  (base + offset) + index);
		printf("Value =  0x%x \n", data );
	}
	FPGABGD_close( hDev );
}

uint32_t *
FPGABGD_get_buffer( FPGABGD_DEV_HANDLER hDev, uint32_t offset , uint32_t nsize, bool to_dev )
{
	return (uint32_t*)(hDev->fpga_base_virt  + offset);
}

void 
FPGABGD_close(FPGABGD_DEV_HANDLER hDev)
{

	if (munmap((void*)hDev->fpga_base_virt, hDev->size ) == -1)
		printf("Unmapping failed \n");
	
	hDev->fpga_base_virt = 0;
	close( hDev->fd );
	free( hDev );
}


void xil_spi_sleep()
{
	usleep( 10 );
}

int xil_spi_init_txrx( FPGABGD_DEV_HANDLER hDev, uint32_t dir_base_addr )
{
	uint32_t cr = 0;
	cr = ( XSP_CR_MASTER_MODE_MASK | XSP_CR_ENABLE_MASK );
	cr |= XSP_CR_TRANS_INHIBIT_MASK;
	cr |= XSP_CR_TXFIFO_RESET_MASK | XSP_CR_RXFIFO_RESET_MASK;

	FPGABGD_WB_write_32( hDev, dir_base_addr | XSP_CR_OFFSET, cr );
	// activate bit 0 in SSR register to select slave 0
	FPGABGD_WB_write_32( hDev, dir_base_addr | XSP_SSR_OFFSET, CS_PLL );
	xil_spi_sleep();

	return 0;
}

int xil_spi_txrx( FPGABGD_DEV_HANDLER hDev, uint32_t dir_base_addr, uint32_t in, uint32_t *out ){
	//uint32_t val_read = 0;
	uint32_t cr, read_value;
	int i;
	//uint8_t  value;
	// Last argument = 0 to Write || fisrt argument = 1 to read

	xil_spi_init_txrx(hDev, dir_base_addr_qspi);

	FPGABGD_WB_write_32( hDev, dir_base_addr_qspi | XSP_DTR_OFFSET, ( uint8_t )(( in >> 16 ) & 0xFF ));
	FPGABGD_WB_write_32( hDev, dir_base_addr_qspi | XSP_DTR_OFFSET, ( uint8_t )(( in >> 8  ) & 0xFF ));
	FPGABGD_WB_write_32( hDev, dir_base_addr_qspi | XSP_DTR_OFFSET, ( uint8_t )( in & 0xFF ));

	xil_spi_sleep();
	//Ask for write transfer (enable=1, inihibit=0)
	cr  = FPGABGD_WB_read_32( hDev, dir_base_addr | XSP_CR_OFFSET );

	cr |= XSP_CR_ENABLE_MASK;
	cr &= ~XSP_CR_TRANS_INHIBIT_MASK;
	FPGABGD_WB_write_32( hDev, dir_base_addr | XSP_CR_OFFSET, cr);


	uint32_t temp = FPGABGD_WB_read_32( hDev, dir_base_addr | XSP_SR_OFFSET );
    //Pool TX completed
	while ((!(temp & XSP_SR_TX_EMPTY_MASK)))
	{
		temp = FPGABGD_WB_read_32( hDev, dir_base_addr | XSP_SR_OFFSET );
	}

	xil_spi_sleep();

	//Clear transfer
	//cr  = FPGABGD_WB_read_32( hDev, dir_base_addr | XSP_CR_OFFSET );
	//cr |= XSP_CR_TRANS_INHIBIT_MASK;
	//FPGABGD_WB_write_32( hDev, dir_base_addr | XSP_CR_OFFSET, cr );

	xil_spi_sleep();

	if( out ){
		//Read receiver FIFO until it is empty
		i=2;
		*out = 0;
		while (!( FPGABGD_WB_read_32(hDev, dir_base_addr | XSP_SR_OFFSET ) & XSP_SR_RX_EMPTY_MASK ))
		{
			read_value = FPGABGD_WB_read_32( hDev, dir_base_addr | XSP_DRR_OFFSET );
			*out |= ((read_value & 0xFF) << 8*i);
			i--;
			
		}
		*out &= 0xFFFFFF;
    }
	return 0;
}

/*
 * mode == 0: write value
 * mode == 1: read value
 */
uint32_t fill_bits_sequence( int mode, uint8_t dir, uint16_t value_to_write ){

	uint32_t in = 0;
	if ( mode ){
		in = in | (dir << 16);		// Adding dir
		in = in | READ_OP;         // read operation
	}
	else{
		in = in | (dir << 16);		// Adding dir
		in = in | value_to_write;
	}
	return in;
}


void info_function (char *s){
    printf("%s\n", s);
    char str[100];
    strcpy(str,"echo '[PLL_programmer]: ");
    strcat(str,s);
    strcat(str, "' > /dev/kmsg");
    system(str);
}

bool 
FPGABGD_isValid( FPGABGD_DEV_HANDLER hDev ) 
{
	return hDev!=NULL;
}

FPGABGD_DEV_HANDLER 
FPGABGD_open( uint64_t base, uint32_t size )
{
	FPGABGD_DEV_HANDLER hDev = 0;
	int fd;
	void *_fpga_base_virt;

	fd = open( "/dev/mem", O_RDWR | O_SYNC );
	if( fd < 0 ){
		fprintf( stderr,
			 "Error while opening device: %s\n Error: %s",
			 "/dev/mem",
			 strerror( errno ));
		hDev->fd = fd;		
		return hDev;
	}

	_fpga_base_virt = mmap( NULL, size, ( PROT_WRITE | PROT_READ | MAP_PRIVATE ), MAP_SHARED, fd, base );

	if( _fpga_base_virt == MAP_FAILED )
	{
		int lerrno = errno;
		fprintf( stderr,
			 "Map failed: Error: %s\n",
			 strerror( lerrno ));		
		close( fd );		
		hDev->fd = -1;
		return hDev;
	}

	hDev = (FPGABGD_DEV_HANDLER)malloc(sizeof( FPGABGD_DEV_STRUCT ));
	set_filedescriptor( hDev, fd );
	hDev->fpga_base_virt = _fpga_base_virt;
	hDev->size = size;	
	hDev->physical_buffer_address = base;
	return hDev;
}


void 
set_filedescriptor( FPGABGD_DEV_HANDLER hDev, int const _fd ){
	hDev->fd = _fd;	
}

int 
get_filedescriptor( FPGABGD_DEV_HANDLER hDev ){
	return hDev->fd;
}

int main(int argc, char *argv[]){

	FILE * fptr;
	char filename[512];
	int i = 0, reg_num, value;
	char c;
	uint32_t in, read_value;
	FPGABGD_DEV_HANDLER spi_pll = 0; //Map SPI bus

	printf("STARTING LMX2594 CONFIGURATION\n");
	if (argc == 1) //Default configuration file
	{
		strcpy(filename, "/wr/lmx2594.conf");
		printf("Configuration file not specified. Using default(%s)\n",filename);
	}
	else if (argc == 2) //Configuration file specified
	{
		strcpy(filename, argv[1]);
		printf("Trying to open %s\n", filename);
	}

	if(access(filename, F_OK)) // File doesn't exist
	{
		printf("File %s doesn't exist\n", filename);
		return -ENOENT;
	}

	fptr = fopen(filename,"r"); // Open the file in read only mode

	for(i=0; i < lmx2594_register_number; i++) // Store values in the struct
	{
		fscanf(fptr, "%c %d %08x\n", &c, &reg_num, &value);
		lmx2594_config[i].reg = reg_num;
		lmx2594_config[i].val = value & 0xFFFF;
	}

	fclose(fptr); // Close the file

	spi_pll = FPGABGD_open( dir_base_addr_qspi, PERIPH_MAPPING_SIZE );
	if( spi_pll->fd < 0 ){
		printf("Error while opening QSPI");
		return -1;
	};

	printf("Resetting LMX\n");

	in = fill_bits_sequence(MODE_WRITE, RESET_REGISTER, RESET_LMX); // Program RESET = 1 to reset registers
	xil_spi_txrx( spi_pll, dir_base_addr_qspi, in, NULL );
	in = fill_bits_sequence(MODE_WRITE, RESET_REGISTER, 0); //Program RESET = 0 to remove reset
	xil_spi_txrx( spi_pll, dir_base_addr_qspi, in, NULL );

	for(i=0; i < lmx2594_register_number ; i++) // Write registers in reverse order
	{
		//in = fill_bits_sequence(MODE_READ, lmx2594_config[i].reg , 0x00 );
		//xil_spi_txrx( spi_pll, dir_base_addr_qspi, in, &read_value );


		in = fill_bits_sequence(MODE_WRITE, lmx2594_config[i].reg, lmx2594_config[i].val);
		xil_spi_txrx( spi_pll, dir_base_addr_qspi, in, NULL );
		//in = fill_bits_sequence(MODE_READ, lmx2594_config[i].reg , 0x00 );
		//xil_spi_txrx( spi_pll, dir_base_addr_qspi, in, &read_value );
	}

	usleep(10000); //Wait 10 ms

	in = fill_bits_sequence(MODE_READ, RESET_REGISTER , 0x00 );	//Read register 0
	xil_spi_txrx(spi_pll, dir_base_addr_qspi, in, &read_value );

	in = FCAL_EN; //Set FCAL_EN bit

	in = fill_bits_sequence(MODE_WRITE, RESET_REGISTER , FCAL_EN | read_value);
	xil_spi_txrx(spi_pll, dir_base_addr_qspi, in, NULL );

	
	return 0;
}
